//******************************************************************
//* MATRIX ORBITAL LCD PANEL DRIVER ROUTINES
//* WRITTEN BY FRED EADY FOR NUTS AND VOLTS MAGAZINE
//* LAST UPDATED 09/30/2008
//* CHANGES/ADDITIONS
//******************************************************************b
#include <pic18.h>
#include <string.h>
#include <stdio.h>
#include <ctype.h>
#include <htc.h>

// Settings for PIC18F6680
__CONFIG(1,HS);
__CONFIG(2,BORDIS & PWRTDIS & WDTDIS);
__CONFIG(3,MCLREN);
__CONFIG(4,DEBUGEN & LVPDIS & STVREN);
__CONFIG(5,UNPROTECT);
__CONFIG(6,WRTEN);
/*
#include <pic18.h>
#include <string.h>
#include <stdio.h>
#include <ctype.h>
#include <htc.h>
#include <stdlib.h>
#include <conio.h>

// Settings for PIC18F2620
__CONFIG(1,IESODIS & FCMDIS & HS);
__CONFIG(2,BORDIS & PWRTDIS & WDTDIS);
__CONFIG(3,MCLREN & LPT1DIS & PBDIGITAL);
__CONFIG(4,DEBUGEN & LVPDIS & XINSTDIS & STVREN);
__CONFIG(5,UNPROTECT);
__CONFIG(6,WRTEN);
*/
__IDLOC(1001);
//******************************************************************
//*	FUNCTION PROTOTYPES
//******************************************************************
void putch(char c);
char recvchar(void);
char sendchar(char data);
char CharInQueue(void);
void display_data(void);
void cls_lcd(void);
void display_image(char image_id);
void cls(void);
//******************************************************************
//*	PIC18F6680 RAM Definitions
//******************************************************************
char bytein,return_addr,physical_addr,idle_clock,rc;
float tempF;
unsigned int isecs1,tmsecs1,tmsecs3,imsecs1,imsecs3,tsecs1,isecs3,imins3;
unsigned int temp16;
#define		act_led			LATC0
//******************************************************************
//*	EUSART BUFFER DEFINITIONS
//******************************************************************
#define BAUD	19200		//desired baud rate
#define FOSC	20000000	//oscillator frequency
#define DIVIDER ((unsigned int)(FOSC/(16 * BAUD) - 1))
//1,2,4,8,16,32,64,128 or 256 bytes 
#define USART_RX_BUFFER_SIZE  256     
#define USART_RX_BUFFER_MASK ( USART_RX_BUFFER_SIZE - 1 )
// 1,2,4,8,16,32,64,128 or 256 bytes 
#define USART_TX_BUFFER_SIZE  256      
#define USART_TX_BUFFER_MASK ( USART_TX_BUFFER_SIZE - 1 )
char USART_RxBuf[USART_RX_BUFFER_SIZE],USART_TxBuf[USART_TX_BUFFER_SIZE];
char USART_TxHead,USART_TxTail,USART_RxHead,USART_RxTail;
//******************************************************************
//*	INTERRUPT DEFINITIONS
//******************************************************************
#define		disable_RXint	  	RCIE = 0
#define		enable_RXint	  	RCIE = 1
#define		disable_TMR2int  	TMR2IE = 0
#define		enable_TMR2int	 	TMR2IE = 1
#define		disable_TMR3int  	TMR3IE = 0
#define		enable_TMR3int	 	TMR3IE = 1
#define		disable_TMR1int  	TMR1IE = 0
#define		enable_TMR1int	 	TMR1IE = 1
#define		disable_TMR0int  	TMR0IE = 0
#define		enable_TMR0int	 	TMR0IE = 1

#define		enable_GLOBALint  	IPEN = 0;				\
							  	GIE = 1;				\
							  	PEIE = 1;
#define		disable_GLOBALint 	IPEN = 0;				\
							  	GIE = 0;				\
							  	PEIE = 0;
#define		TIMER0ON			TMR0ON = 1
#define		TIMER0OFF			TMR0ON = 0
#define		TIMER1ON			TMR1ON = 1
#define		TIMER1OFF			TMR1ON = 0
#define		TIMER2ON		  	TMR2ON = 1
#define		TIMER2OFF		  	TMR2ON = 0
#define		TIMER3ON		  	TMR3ON = 1
#define		TIMER3OFF		  	TMR3ON = 0

#define mdelay1(msecdelay)		TIMER1OFF;					\
								TMR1IF = 0;					\
								imsecs1 = 0;				\
								tmsecs1 = 0;				\
								TIMER1ON;					\
								while(tmsecs1 < msecdelay);
								
#define	sdelay1(secdelay)		TIMER1OFF;					\
								TMR1IF = 0;					\
								imsecs1 = 0;				\
								tmsecs1 = 0;				\
								isecs1 = 0;					\
								tsecs1 = 0;					\
								TIMER1ON;					\
								while(tsecs1 < secdelay);

void putch(char c)
{
   while(!TRMT) 	// TRMT is set when TSR is empty
      continue;

  TXREG = c;		// load the register
}

//******************************************************************
//*	INTERRUPT HANDLER ROUTINE
//******************************************************************
void interrupt TIMERS(void)
{
  char usart_data,tmphead,tmptail;
	if((TMR3IF && TMR3IE))
	{
		TMR3IF = 0;
		TMR3H = 0xEC;	
		TMR3L = 0x77;
		++tmsecs3;
		 if(++imsecs3 == 1000)
         {
          imsecs3 = 0;
          ++isecs3;
 		  act_led ^= 1;
        }
	    if(isecs3 == 60)
	     {
			isecs3 = 0;
			++imins3;
		 }
	}

	if((TMR1IF && TMR1IE))
	{
		TMR1IF = 0;
		TMR1H = 0xEC;	
		TMR1L = 0x77;

		++tmsecs1;
		if(++imsecs1 == 1000)
		{
			imsecs1 = 0;
			++tsecs1;
		}
	}

  if(RCIF)
 {
   usart_data = RCREG;                   // read the received usart_data 
                                // calculate buffer index 
   tmphead = ( USART_RxHead + 1 ) & USART_RX_BUFFER_MASK;
   USART_RxHead = tmphead;        // store new index 

   if ( tmphead == USART_RxTail )
   {
     // ERROR! Receive buffer overflow 
   }
                                
   USART_RxBuf[tmphead] = usart_data;   // store received usart_data in buffer 
  }

  
  if(TRMT)
  {
                                // check if all usart_data is transmitted  
   if ( USART_TxHead != USART_TxTail )
   {
                                // calculate buffer index  
     tmptail = ( USART_TxTail + 1 ) & USART_TX_BUFFER_MASK;
     USART_TxTail = tmptail;      // store new index  

    TXREG = USART_TxBuf[tmptail];  // start transmitiON  
   }
   else
   {
     TXIE = 0;         // disable TX interrupt  
   }
  }
} 
//******************************************************************
//*	Init EUSART Function
//******************************************************************
void init_EUSART(void)
{
// load baud rate divisor for 19200 bps with 20MHz clock  
  SPBRG = 64;	
  TRISC7 = 1;		//receive pin
  TRISC6 = 0;		//transmit pin
  TXSTA = 0x04;		//high speed baud rate	
  RCSTA = 0x80;		//enable serial port and serial port pins
  
  USART_RxTail = 0x00;	//flush receive buffer 
  USART_RxHead = 0x00;
  USART_TxTail = 0x00;	//flush transmit buffer
  USART_TxHead = 0x00;
  RCIP = 1;			//receive interrupt = high priority
  TXIP = 1;			//transmit interrupt = high priority
  RCIE = 1;			//enable receive interrupt
  PEIE = 1;			//enable all unmasked peripheral interrupts
  GIE = 1;			//enable all unmasked interrupts
  CREN = 1;			//enable USART1 receiver
  TXIE = 0;			//disable USART1 transmit interrupt
  TXEN = 1;			//transmitter enabled
}
//****************************************************************************
//*	INITIALIZE ROUTINE
//****************************************************************************
void init(void)
{
	char i;

   	TRISA = 0b11111111;		
   	TRISB = 0b11110000;		
   	TRISC = 0b10010010;
//******************************************************************
//*	CONFIGURE EEPROM READ/WRITE
//******************************************************************
	EECON1 = 0b00000000;
//******************************************************************
//*	CONFIGURE A2D AND COMPARATORS
//******************************************************************
	ADCON0 = 0b00000000;	//turn OFF A2D
	ADCON1 = 0b00001111;	//all digital
//******************************************************************
//*	CONFIGURE AND START TIMER1
//* SET TO OVERFLOW EVERY 1mS
//******************************************************************
	TIMER1OFF;
	T1CON = 0b00000000;
	TMR1H = 0xEC;
	TMR1L = 0x77;
	TIMER1ON;
//******************************************************************
//*	CONFIGURE AND START TIMER3
//* SET TO OVERFLOW EVERY 1mS
//******************************************************************
	imins3 = 0x00;
	isecs3 = 0x00;
  	imsecs3 = 0;
	T3CON = 0x00;		
	TMR3H = 0xEC;	
	TMR3L = 0x77;
	TIMER3ON;
//******************************************************************
//*	INITIALIZE THE EUSART
//******************************************************************
	init_EUSART();
//******************************************************************
//*	ACTIVATE INTERRUPTS
//******************************************************************
	enable_TMR1int;
	enable_TMR3int;
    enable_GLOBALint;
//******************************************************************
}
char recvchar(void)
{
  char tmptail;
                                // wait for incomming usart_data  
  while ( USART_RxHead == USART_RxTail );
                                // calculate buffer index  
  tmptail = ( USART_RxTail + 1 ) & USART_RX_BUFFER_MASK;
  USART_RxTail = tmptail;        // store new index  

  return USART_RxBuf[tmptail];   // return usart_data  
}

//******************************************************************
//*	SEND A CHARACTER
//******************************************************************
char sendchar(char usart_data)
{
  char tmphead;
                                // calculate buffer index 
  tmphead = ( USART_TxHead + 1 ) & USART_TX_BUFFER_MASK;
                                // wait for free space in buffer 
  while ( tmphead == USART_TxTail );
                                // store usart_data in buffer 
  USART_TxBuf[tmphead] = usart_data;
  USART_TxHead = tmphead;        // store new index 

  TXIE = 1;            // enable TX interrupt 

  return usart_data;
}

char CharInQueue(void)
{
  return(USART_RxHead != USART_RxTail);
}
//**************************************************************
//*  DISPLAY SOME TEXT USING PRINTF
//**************************************************************
void display_text(void)
{
	printf("%c%c%c%c NUTS AND VOLTS",0xFE,0x47,1,1);
}
//**************************************************************
//*  DISPLAY SOME TEXT USING SENDCHAR
//**************************************************************
void display_text_sendchar(void)
{
	rc = sendchar('N');
	rc = sendchar('U');
	rc = sendchar('T');
	rc = sendchar('S');
	rc = sendchar(' ');
	rc = sendchar('A');
	rc = sendchar('N');
	rc = sendchar('D');
	rc = sendchar(' ');
	rc = sendchar('V');
	rc = sendchar('O');
	rc = sendchar('L');
	rc = sendchar('T');
	rc = sendchar('S');
}
//**************************************************************
//*  CLEAR THE SCREEN
//**************************************************************
void clear_screen(void)
{
	rc = sendchar(0xFE);
	rc = sendchar(0x58);
}
//**************************************************************
//*  HOME THE CURSOR
//**************************************************************
void home_cursor(void)
{
	rc = sendchar(0xFE);
	rc = sendchar(0x48);
}
//**************************************************************
//*  SET THE CURSOR POSITION
//**************************************************************
void position_cursor(char column,char row)
{
	rc = sendchar(0xFE);
	rc = sendchar(0x47);
	rc = sendchar(column);
	rc = sendchar(row);
}
//**************************************************************
//*  CLEAR SCREEN
//**************************************************************
void cls_lcd(void)
{	
	printf("%c%c",0xFE,0x58);
}
//**************************************************************
//*  DISPLAY STORED IMAGE
//**************************************************************
void display_image(char image_id)
{  	
  	printf("%c%c",0xFE,0x58);
	printf("%c%c%c%c%c",0xFE,0x62,image_id,0x01,0x01);
}
//**************************************************************
//*  DISPLAY TANK TEMPERATURES
//**************************************************************
void display_data(void)
{
	cls_lcd();
	printf("%c%c%c%c NUTS AND VOLTS / DESIGN CYCLE",0xFE,0x47,3,1);
	printf("%c%c%c%c TANK1",0xFE,0x47,1,5);
	printf("%c%c%c%c TANK2",0xFE,0x47,1,6);
	printf("%c%c%c%c TANK3",0xFE,0x47,1,7);
	printf("%c%c%c%c TANK4",0xFE,0x47,1,8);
	printf("%c%c%c%c SYS_A",0xFE,0x47,7,3);
	printf("%c%c%c%c SYS_B",0xFE,0x47,14,3);
	printf("%c%c%c%c SYS_C",0xFE,0x47,21,3);
	
	tempF = (1.8 * 100) + 32;
	printf("%c%c%c%c %3.2f",0xFE,0x47,7,5,tempF);
	printf("%c%c%c%c %3.2f",0xFE,0x47,14,5,tempF);
	printf("%c%c%c%c %3.2f",0xFE,0x47,21,5,tempF);
	mdelay1(100);
	tempF = (1.8 * 110) + 32;
	printf("%c%c%c%c %3.2f",0xFE,0x47,7,6,tempF);
	printf("%c%c%c%c %3.2f",0xFE,0x47,14,6,tempF);
	printf("%c%c%c%c %3.2f",0xFE,0x47,21,6,tempF);
	mdelay1(100);
	tempF = (1.8 * 120) + 32;
	printf("%c%c%c%c %3.2f",0xFE,0x47,7,7,tempF);
	printf("%c%c%c%c %3.2f",0xFE,0x47,14,7,tempF);
	printf("%c%c%c%c %3.2f",0xFE,0x47,21,7,tempF);
	mdelay1(100);
	tempF = (1.8 * 130) + 32;
	printf("%c%c%c%c %3.2f",0xFE,0x47,7,8,tempF);
	printf("%c%c%c%c %3.2f",0xFE,0x47,14,8,tempF);
	printf("%c%c%c%c %3.2f",0xFE,0x47,21,8,tempF);
	mdelay1(100);

}


void main(void)
{
//******************************************************************
//*	INITIALIZE
//******************************************************************
  	init();				//init the EUSART and TIMERS
  	sdelay1(2);
  	display_image(0x01);
	sdelay1(30);
  	display_data();
  	sdelay1(30);
  	while(1);


}
